#!/bin/sh

#Script for disabling popup windows which are mounted from idrac partitions 

# Global constants - readonly variables
typeset -ri _REDHAT4=0
typeset -ri _REDHAT5=1
typeset -ri _SUSE10=3
typeset -ri _SUSE11=7
typeset -ri _REDHAT6=10
typeset -ri _CENTOS=12

# required files - constant
typeset -r RHatUdevLibDir="/etc/udev/rules.d/"
typeset -r RHEL5Rules="00-secupd-dell.rules"
typeset -r RHEL6Rules="98-secupdusb.rules"

typeset -r COPY="cp -f"
typeset -r REMOVE="rm -f"
typeset -r GREP="grep"
typeset -r TOUCH="touch"
typeset -r GREPCaseInsArgs="-i"
typeset -r GREPCaseInsAndOnlyOutuptArgs="-io"
typeset -r AWK="awk"
typeset -r CAT="cat"
typeset	-r UdevadmCmd="/sbin/udevadm"
typeset -r UdevadmCmdArgs="control --reload-rules"
typeset	-r UdevcontrolCmd="/sbin/udevcontrol"
typeset	-r UdevcontrolCmdArgs="reload_rules"

typeset -r HALDaemonCmd="/etc/init.d/haldaemon"
typeset -r HALDaemonCmdStartArgs="start"
typeset -r HALDaemonCmdStopArgs="stop"
typeset -r HALDaemonCmdStatusArgs="status"
typeset -r HalDaemonTempFile="haldaemonrunning"

typeset	-r RedHatRelaseFile="/etc/redhat-release"
typeset -r RedHatSearchString="release [0-9*]"

typeset -r SuSeReleaseFile="/etc/SuSE-release"
typeset -r SuseSearchString="version"

typeset	LinuxOSVer=-1

RestartUdevRules()
{
	echo "Entering Restart UdevRules function"	
#	if [ -e $UdevcontrolCmd -a -x $UdevcontrolCmd]; then
	if which $UdevcontrolCmd > /dev/null 2>&1; then
		$UdevcontrolCmd $UdevcontrolCmdArgs   > /dev/null 2>&1
#	elif [ -e $UdevadmCmd -a -x $UdevadmCmd ]; then
	elif which $UdevadmCmd > /dev/null 2>&1 ; then
		$UdevadmCmd $UdevadmCmdArgs   > /dev/null 2>&1
	else
		echo "Failed to restart udev rules"
	fi
	echo "Exiting Restart UdevRules"
}

StartHalDaemon()
{
	echo "Entering StartHALDaemon function"
#	if [ -e $HALDaemonCmd -a -x $HALDaemonCmd ]; then
	if which $HALDaemonCmd > /dev/null 2>&1 ; then
		$HALDaemonCmd $HALDaemonCmdStartArgs   > /dev/null 2>&1
	else
		echo "Failed to start HalDaemon"
	fi
	echo "Exiting StartHALDaemon function"
}

StopHalDaemon()
{
	
	echo "Entering Stoping HALDaemon"
#	if [ -e $HALDaemonCmd -a -x $HALDaemonCmd ]; then
	if which $HALDaemonCmd > /dev/null 2>&1 ; then
		$HALDaemonCmd $HALDaemonCmdStopArgs   > /dev/null 2>&1
	else
		echo "Failed to stop HalDaemon"
	fi
	echo "Exiting StopHALDaemon function"
}

IsHalDaemonRunning()
{
	local HalDaemonRunningStatus
	echo "Entering IsHalDaemonRunning function"	
#	if [ -e $HALDaemonCmd -a -x $HALDaemonCmd ]; then
	if which $HALDaemonCmd > /dev/null 2>&1 ; then
		$HALDaemonCmd $HALDaemonCmdStatusArgs   > /dev/null 2>&1
	if [ $? -eq 0 ]; then
		$TOUCH $HalDaemonTempFile   > /dev/null 2>&1
	fi
	else
		echo "Failed to check the status of HalDaemon"
	fi
	echo "Exiting IsHalDaemonRunning function"	
}

CopyFile()
{
	echo "Entering CopyFile function"
	local source=$1
	local destination=$2
	if [ -e $source -a -d $destination ]; then
		$COPY $source $destination   > /dev/null 2>&1
		if [ $? -ne 0 ]; then
			echo "Copy Rules files to udev directory failed"
		fi
	else
		echo "Could not able to found rules file or udev directory"
	fi
	echo "Exiting CopyFile function"
}

RemoveFile()
{
	echo "Entering RemoveFile function"
	local source=$1
	if [ -e $source ]; then
		$REMOVE $source   > /dev/null 2>&1
		if [ $? -ne 0 ]; then
			echo "Could not able ro remove the rules file from udev directory"
		fi
	else
		echo "Could not able to delete the copied rules file from udev directory"
	fi
	echo "Exiting RemoveFile function"
}

getLinuxOSVersion()
{
	echo "Entering getLinuxOSVersion function"
	local redHatRelease
	local SuSERelease
	if [ -e $RedHatRelaseFile ]; then
		redHatRelease=`$CAT $RedHatRelaseFile | $GREP $GREPCaseInsAndOnlyOutuptArgs "$RedHatSearchString" | $AWK '{print $2}'`   > /dev/null 2>&1
		if [ $? -ne 0 ]; then
			echo "Could not able to found the red hat version from release file $RedHatRelaseFile"
		fi
		case $redHatRelease in
			4)
				LinuxOSVer=$_REDHAT4
				;;
			5)	
				LinuxOSVer=$_REDHAT5
				;;
			6)	
				LinuxOSVer=$_REDHAT6	
				;;
			*)	
				LinuxOSVer=-1
				;;
		esac
	elif [ -e $SuSeReleaseFile ]; then
		SuSERelease=`$CAT $SuSeReleaseFile | $GREP $GREPCaseInsArgs $SuseSearchString | $AWK '{print$3}'` > /dev/null 2>&1
		if [ $? -ne 0 ]; then
			echo "Could not able to found the SUSE version from release file $SuSeReleaseFile"
		fi
		case $SuSERelease in
			9)
				LinuxOSVer=$_SUSE9
				;;
			10)	
				LinuxOSVer=$_SUSE10
				;;
			11)	
				LinuxOSVer=$_SUSE11
				;;
			*)	
				LinuxOSVer=-1
				;;
		esac

	else
		LinuxOSVer=-1
		echo "Unknown version of Linux found" > /dev/null 2>&1
	fi
	echo "Exiting getLinuxOSVersion function"

}

enablePopUp()
{
	echo "Entering enablePopUp function"
	getLinuxOSVersion
	echo "Found LinuxOSVersion during halinstall is $LinuxOSVer"
	case $LinuxOSVer in
                        $_REDHAT4 | $_REDHAT5)
						RemoveFile $RHatUdevLibDir/$RHEL5Rules
						RestartUdevRules
						;;
                      	$_REDHAT6)
						RemoveFile $RHatUdevLibDir/$RHEL6Rules
						RestartUdevRules
						;;
			$_SUSE10 | $_SUSE11)
						IsHalDaemonRunning
						if [ -e $HalDaemonTempFile ]; then
							StartHalDaemon
							$REMOVE $HalDaemonTempFile
						fi
						;;
                        *)
                                echo "Unknow Linux OS Found during udevhalUnInstall"
                                ;;
	esac
	echo "Exiting enablePopUp function"
}

disablePopUp()
{
	echo "Entering disablePopUp function"
	getLinuxOSVersion	
	echo "Found LinuxOSVersion $LinuxOSVer"
	case $LinuxOSVer in
                        $_REDHAT4 | $_REDHAT5)
						CopyFile $RHEL5Rules $RHatUdevLibDir
						RestartUdevRules
						;;
                      	$_REDHAT6)
						CopyFile $RHEL6Rules $RHatUdevLibDir
						RestartUdevRules
						;;
			$_SUSE10 | $_SUSE11)
						IsHalDaemonRunning
						if [ -e $HalDaemonTempFile ]; then
							StopHalDaemon
						fi
						;;
                        *)
                                echo "Unknow Linux OS Found during udevhalInstall"
                                ;;
	esac
	echo "Exiting disablePopUp function"
}

usage()
{
	echo "Usage:"
	echo "To disable popups use:"
	echo "`basename $0` disablepopup"
	echo "To enable popups use:"
	echo "`basename $0` enablepopup"
}


#Start of Main Script
if   [ "$1" == "usage" ]; then
	usage
elif [ "$1" == "disablepopup" ]; then
	disablePopUp
elif [ "$1" == "enablepopup" ]; then
	enablePopUp
else 
	usage
fi
